Unit MNA3R_p;
interface
uses
SysUtils, Math, { Delphi }
Lstruct, Lfunc;

procedure MNA3R(F :Func_F1; var A :Real; var B :Real; var C :Real;
                EPS :Real; var XMIN :Real; var FMIN :Real;
                var IFLAG :Integer; ITMAX :Integer);

implementation

procedure MNA3R(F :Func_F1; var A :Real; var B :Real; var C :Real;
                EPS :Real; var XMIN :Real; var FMIN :Real;
                var IFLAG :Integer; ITMAX :Integer);
var
I :Integer;
AX,BX,V,W,X,E,FX,FV,FW,TOL1,TOL2,XM,R,Q,P,ETEMP,D,U,FU,CX :Real;
const
CGOLD :Real = 0.3819660;
ZEPS :Real = 1.0E-10;
ZERO :Real = 0.0;
label
_10,_20,_30,_40,_50,_60,_70,_80,_90,_100,_110,_120,_130,_140,_150,
_160;
begin
IFLAG := 0;
AX := A;
BX := B;
CX := C;
A := EMin1(A,C);
B := EMax1(A,C);
V := BX;
W := V;
X := V;
E := ZERO;
FX := F(X);
FV := FX;
FW := FX;
for I:=1 to ITMAX do
 begin
  _MNA3RR.elm1 := I;
  XM := 0.5*(A+B);
  TOL1 := EPS*Abs(X)+ZEPS;
  TOL2 := 2.0*TOL1;
  if ( Abs(X-XM) <= (TOL2-0.5*(B-A)) ) 
   then goto _150;
  if ( Abs(E) <= TOL1 ) 
   then goto _10;
  R := (X-W)*(FX-FV);
  Q := (X-V)*(FX-FW);
  P := (X-V)*Q-(X-W)*R;
  Q := 2.0*(Q-R);
  if ( Q > ZERO ) 
   then P := -P;
  Q := Abs(Q);
  ETEMP := E;
  E := D;
  if ( Abs(P) >= Abs(0.5*Q*ETEMP) ) or ( P <= Q*(A-X) ) or ( P >= Q*(B-X) )
   then goto _10;
  D := P/Q;
  U := X+D;
  if ( U-A < TOL2 ) or ( B-U < TOL2 ) 
   then D := Sign(TOL1,XM-X);
  goto _40;
_10:
  if ( X < XM ) 
   then goto _20;
  E := A-X;
  goto _30;
_20:
  E := B-X;
_30:
  D := CGOLD*E;
_40:
  if ( Abs(D) < TOL1 ) 
   then goto _50;
  U := X+D;
  goto _60;
_50:
  U := X+Sign(TOL1,D);
_60:
  FU := F(U);
  if ( FU > FX ) 
   then goto _90;
  if ( U < X ) 
   then goto _70;
  A := X;
  goto _80;
_70:
  B := X;
_80:
  V := W;
  FV := FW;
  W := X;
  FW := FX;
  X := U;
  FX := FU;
  goto _110;
_90:
  if ( U >= X ) 
   then goto _100;
  A := U;
  goto _110;
_100:
  B := U;
_110:
  if ( FU <= FW ) or ( W = X ) 
   then goto _120;
  if ( FU <= FV ) or ( V = X ) or ( V = W ) 
   then goto _130;
  goto _140;
_120:
  V := W;
  FV := FW;
  W := U;
  FW := FU;
  goto _140;
_130:
  V := U;
  FV := FU;
_140:
 end;
goto _160;
_150:
IFLAG := 1;
_160:
A := AX;
B := BX;
C := CX;
XMIN := X;
FMIN := FX;
end;

end.
